@@ -2,7 +2,7 @@
 """A framework for conducting multi-strand multilateral asynchronous negotiations on multiple issues."""
 __author__ = """Yasser Mohammad"""
 __email__ = "yasserfarouk@gmail.com"
-__version__ = "0.6.10"
+__version__ = "0.6.12"
 
 from .common import *
 from .config import *
@@ -30,5 +30,5 @@ __all__ = (
     + genius.__all__
     + situated.__all__
     + modeling.__all__
-    + ["generics", "helpers", "events", "tournaments"]
+    + ["generics", "helpers", "events", "tournaments", "elicitation"]
 )
