@@ -4,27 +4,17 @@ Implements Stacked Alternating Offers (SAO) mechanism.
 import itertools
 import math
 import random
+import sys
 import time
+from typing import Any, Dict, List, Optional, Tuple, Union
 
 import pandas as pd
-from typing import (
-    Any,
-    Dict,
-    List,
-    Optional,
-    Tuple,
-    Union,
-)
 
 from .common import SAOResponse, SAOState
-from ..common import *
+from ..common import AgentMechanismInterface, MechanismState
 from ..events import Event
 from ..mechanisms import Mechanism, MechanismRoundResult
-from ..outcomes import (
-    Outcome,
-    ResponseType,
-    outcome_is_complete,
-)
+from ..outcomes import Outcome, ResponseType, outcome_is_complete
 from ..utilities import UtilityFunction
 
 __all__ = [
@@ -108,6 +98,7 @@ class SAOMechanism(Mechanism):
         ignore_negotiator_exceptions=False,
         offering_is_accepting=True,
         name: Optional[str] = None,
+        max_wait: int = sys.maxsize,
         **kwargs,
     ):
         """
@@ -145,6 +136,8 @@ class SAOMechanism(Mechanism):
         self._no_responses = 0
         self._new_offers = []
         self._offering_is_accepting = offering_is_accepting
+        self._n_waits = 10
+        self._n_max_waits = max_wait if max_wait is not None else float("inf")
 
     def join(
         self,
@@ -180,7 +173,7 @@ class SAOMechanism(Mechanism):
                 new_offerer_agents.append(agent.id)
             else:
                 new_offerer_agents.append(None)
-        return SAOState(
+        return dict(
             current_offer=self._current_offer,
             new_offers=self._new_offers,
             current_proposer=self._current_proposer.id
@@ -434,10 +427,10 @@ class SAOMechanism(Mechanism):
 
         """
         self._new_offers = []
-        negotiators: List[SAONegotiator] = self.negotiators
+        negotiators: List["SAONegotiator"] = self.negotiators
         n_negotiators = len(negotiators)
 
-        def _safe_counter(negotiator, *args, **kwargs):
+        def _safe_counter(negotiator, *args, **kwargs) -> SAOResponse:
             try:
                 if (
                     negotiator == self._current_proposer
@@ -523,15 +516,6 @@ class SAOMechanism(Mechanism):
                     return MechanismRoundResult(
                         broken=False, timedout=True, agreement=None
                     )
-                if self._enable_callbacks:
-                    for other in self.negotiators:
-                        if other is not neg:
-                            other.on_partner_response(
-                                state=self.state,
-                                agent_id=neg.id,
-                                outcome=None,
-                                response=resp,
-                            )
                 if resp.response == ResponseType.END_NEGOTIATION:
                     return MechanismRoundResult(
                         broken=True, timedout=False, agreement=None
@@ -583,18 +567,27 @@ class SAOMechanism(Mechanism):
             resp = _safe_counter(neg, state=self.state, offer=self._current_offer)
             if time.perf_counter() - strt > self.ami.step_time_limit:
                 return MechanismRoundResult(broken=False, timedout=True, agreement=None)
+            if resp.response != ResponseType.WAIT:
+                self._n_waits = 0
             if self._enable_callbacks:
-                for other in self.negotiators:
-                    if other is not neg:
-                        other.on_partner_response(
-                            state=self.state,
-                            agent_id=neg.id,
-                            outcome=self._current_offer,
-                            response=resp,
-                        )
+                if self._current_offer is not None:
+                    for other in self.negotiators:
+                        if other is not neg:
+                            other.on_partner_response(
+                                state=self.state,
+                                partner_id=neg.id,
+                                outcome=self._current_offer,
+                                response=resp.response,
+                            )
             if resp.response == ResponseType.NO_RESPONSE:
                 continue
             if resp.response == ResponseType.WAIT:
+                self._n_waits += 1
+                if self._n_waits > self._n_max_waits:
+                    self._n_waits = 0
+                    return MechanismRoundResult(
+                        broken=False, timedout=True, agreement=None, waiting=False
+                    )
                 self._last_checked_negotiator = neg_indx - 1
                 if neg_indx < 0:
                     self._last_checked_negotiator = n_negotiators - 1
@@ -613,7 +606,7 @@ class SAOMechanism(Mechanism):
                 proposal = resp.outcome
                 if proposal is None:
                     if (
-                        neg.capabilities.get("propose", False)
+                        neg.capabilities.get("propose", True)
                         and self.end_negotiation_on_refusal_to_propose
                     ):
                         return MechanismRoundResult(
@@ -630,7 +623,7 @@ class SAOMechanism(Mechanism):
                             if other is neg:
                                 continue
                             other.on_partner_proposal(
-                                agent_id=neg.id, offer=proposal, state=self.state
+                                partner_id=neg.id, offer=proposal, state=self.state
                             )
         return MechanismRoundResult(broken=False, timedout=False, agreement=None)
 
