@@ -4,7 +4,6 @@ import random
 import time
 from copy import deepcopy
 from dataclasses import dataclass
-
 from typing import List, Optional, Tuple, Union
 
 from .mechanisms import Mechanism, MechanismRoundResult, MechanismState
@@ -67,7 +66,7 @@ class VetoSTMechanism(Mechanism):
         """The new offer generated in this step"""
 
     def extra_state(self):
-        return STState(
+        return dict(
             current_offer=deepcopy(self.current_offer),
             new_offer=deepcopy(self.new_offer),
         )
